using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX.DirectInput;
using Microsoft.DirectX;

namespace SputnikDemo
{
   
    public partial class Form1 : Form
    {
        //here is some import parameter depending on what kind of robot you are using
        //here is for X80 robot
        private const double WheelDis = 0.26;
        private const double WheelR = 0.0825;
        private const Int16 CircleCnt = 1200;       //one circle encoder value
        private int MotorDemoStep = 0;
        private int ServoDemoStep = 0;

        public static JoystickState state = new JoystickState();
        private Device applicationDevice = null;
        public static int numPOVs = 0;


        private int BrakeLevel = 10000;
        private int GasLevel = 0;
        private int SteerLevel = 5000;
        private int BackwardLevel = 5000;
        private int GearLevel = 1;
        private Boolean blnJoyStick = false;

        private double leftwheel_PWM = 0;
        private double rightwheel_PWM = 0;

        private const double stallY = 0.01;
        private const int FricL = 6000;
        private const int FricR = 6000;



        //for listen
        private Boolean blnListen = false;
        private WaveLib.WaveOutPlayer m_Player;
        private WaveLib.FifoStream m_Fifo = new WaveLib.FifoStream();

        private byte[] m_PlayBuffer;
        private byte[] m_RecBuffer;


        public Form1()
        {
            InitializeComponent();
            
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            myRobot.connectRobot("motion");
            myCamera.connectRobot("power");
            tmrCamera.Enabled = true;
            if (InitDirectInput())
                tmrJoyStickPoll.Start();

        }

        private bool InitDirectInput()
        {
            // Enumerate joysticks in the system.
            foreach (DeviceInstance instance in Manager.GetDevices(DeviceClass.GameControl, EnumDevicesFlags.AttachedOnly))
            {
                // Create the device.  Just pick the first one
                applicationDevice = new Device(instance.InstanceGuid);
                break;
            }

            if (null == applicationDevice)
            {
                MessageBox.Show("Unable to create a joystick device.", "No joystick found");
                return false;
            }

            // Set the data format to the c_dfDIJoystick pre-defined format.
            applicationDevice.SetDataFormat(DeviceDataFormat.Joystick);
            // Set the cooperative level for the device.
            applicationDevice.SetCooperativeLevel(this, CooperativeLevelFlags.Exclusive | CooperativeLevelFlags.Foreground);
            // Enumerate all the objects on the device.
            foreach (DeviceObjectInstance d in applicationDevice.Objects)
            {
                // For axes that are returned, set the DIPROP_RANGE property for the
                // enumerated axis in order to scale min/max values.

                if ((0 != (d.ObjectId & (int)DeviceObjectTypeFlags.Axis)))
                {
                    // Set the range for the axis.
                    applicationDevice.Properties.SetRange(ParameterHow.ById, d.ObjectId, new InputRange(0, +10000));
                }
                // Update the controls to reflect what
                // objects the device supports.
             //  UpdateControls(d);
            }
            return true;
        }


        private void GetData()
        {
            // Make sure there is a valid device.
            if (null == applicationDevice)
                return;

            try
            {
                // Poll the device for info.
                applicationDevice.Poll();
            }
            catch (InputException inputex)
            {
                if ((inputex is NotAcquiredException) || (inputex is InputLostException))
                {
                    // Check to see if either the app
                    // needs to acquire the device, or
                    // if the app lost the device to another
                    // process.
                    try
                    {
                        // Acquire the device.
                        applicationDevice.Acquire();
                    }
                    catch (InputException)
                    {
                        // Failed to acquire the device.
                        // This could be because the app
                        // doesn't have focus.
                        return;
                    }
                }

            } //catch(InputException inputex)

            // Get the state of the device.
            try { state = applicationDevice.CurrentJoystickState; }
            // Catch any exceptions. None will be handled here, 
            // any device re-aquisition will be handled above.  
            catch (InputException)
            {
                return;
            }

            UpdateControl();

        }
        private void UpdateControl()
        {
            byte[] buttons = state.GetButtons();

            if ((buttons[0] & 0x80) != 0)
                BrakeLevel = 0;
            else
                BrakeLevel = 10000;

            if ((buttons[6] & 0x80) != 0)
                GearLevel = 1;
            else if (( buttons[7] & 0x80) != 0)
                GearLevel = 2;
            else if (( buttons[8] & 0x80) != 0)
                GearLevel = 3;

            int[] slider = state.GetSlider();

            GasLevel = slider[0];

            BackwardLevel = state.Y;
            SteerLevel = state.X;

            lblGasLevel.Text = GasLevel.ToString();
            lblBrakeLevel.Text = BrakeLevel.ToString();
            lblGearLevel.Text = GearLevel.ToString();
            lblBackWardLevel.Text = BackwardLevel.ToString();
            lblSteerLevel.Text = SteerLevel.ToString();

            if (blnJoyStick)
                RobotDrive();
            
        }

        private void RobotDrive()
        {
           
   
            Steeringwheel2PWM();
    
   
    
            SendWheelDataToRobot();
        }
        private void SendWheelDataToRobot()
        {
            short LeftWheelValue = 16383;
            short RightWheelValue = 16383;
            
            LeftWheelValue = (short)(leftwheel_PWM * 16383 + 16383);
            RightWheelValue = (short)(rightwheel_PWM * 16383 + 16384);
    
            if (LeftWheelValue > 32766)
                LeftWheelValue = 32766;
            
            if (LeftWheelValue < 0)
                LeftWheelValue = 0;
    
            if (RightWheelValue > 32766)
                RightWheelValue = 32766;

            if (RightWheelValue < 0)
                RightWheelValue = 0;

            

            myRobot.DcMotorPwmNonTimeCtr (0, LeftWheelValue);
            myRobot.DcMotorPwmNonTimeCtr(1, RightWheelValue);
        }


        private void Steeringwheel2PWM()
        {
           
            double Y =0;
            double z =0;
            int PWMValue =0;

            double PowerR =0;
            int maxFric =0;
    
            if (BackwardLevel < 5500) //forward drive
                Y = ((double)(10000 - GasLevel)) / 10000.0;
            else
                Y = -((double)(10000 - GasLevel)) / 10000.0;
            
    
    
            z = ((double)(SteerLevel - 5000)) / 5000.0 / (1 + (double)GearLevel * 0.2);
        
            PowerR = (double)GearLevel * 0.3333;
            
            maxFric = FricL;
            
    
            PWMValue = (int)(Y * (double)(16383 - maxFric));
  
            if ( (Math.Abs(Y) > stallY) && (BrakeLevel > 7000) && (GearLevel != 0) )
                if (Y >= 0){ //forward
                    if (z <= 0){
                        rightwheel_PWM = 16383 + FricR + (int)((double)PWMValue * PowerR);
                        leftwheel_PWM = 16383 + Math.Sign(0.5 + 0.5 * (1 + 2 * z)) * (double)FricL + PWMValue * PowerR * (0.5 + 0.5 * (1 + 2 * z));
                    }
                    
                    else{
                        rightwheel_PWM = 16383 + Math.Sign(0.5 + 0.5 * (1 - 2 * z)) * FricR + PWMValue * PowerR * (0.5 + 0.5 * (1 - 2 * z));
                        leftwheel_PWM = 16383 + FricL + PWMValue * PowerR;
                    }
                 
                }    
                else{ //backard
                    if (z <= 0){
                        leftwheel_PWM = 16383 - Math.Sign(0.5 + 0.5 * (1 + 2 * z)) * FricL + PWMValue * PowerR * (0.5 + 0.5 * (1 + 2 * z));
                        rightwheel_PWM = 16383 - FricR + PWMValue * PowerR;
                    }
                    else{
                        leftwheel_PWM = 16383 - FricL + PWMValue * PowerR;
                        rightwheel_PWM = 16383 -Math.Sign(0.5 + 0.5 * (1 - 2 * z)) * FricR + PWMValue * PowerR * (0.5 + 0.5 * (1 - 2 * z));
                    }
                
                }
        
            else{
                rightwheel_PWM = 16383;
                leftwheel_PWM = 16383;
            }
    
    
            leftwheel_PWM = -(leftwheel_PWM - 16383) / 16383;
            rightwheel_PWM = (rightwheel_PWM - 16383) / 16383;

           

           
           
        }
        

        private void myRobot_StandardSensorEvent(object sender, EventArgs e)
        {
            lblUSDis1.Text = ((double)(myRobot.GetSensorSonar1()) /100).ToString() + "m";
            lblUSDis2.Text = ((double)(myRobot.GetSensorSonar2()) / 100).ToString() + "m";
            lblUSDis3.Text = ((double)(myRobot.GetSensorSonar3()) / 100).ToString() + "m";
            lblUSDis4.Text = ((double)(myRobot.GetSensorSonar4()) / 100).ToString() + "m";
            lblUSDis5.Text = ((double)(myRobot.GetSensorSonar5()) / 100).ToString() + "m";
            lblUSDis6.Text = ((double)(myRobot.GetSensorSonar6()) / 100).ToString() + "m";

            lblIRDis1.Text = ( AD2Dis(myRobot.GetSensorIRRange())).ToString("0.00") + "m";
            lblBoardVol.Text = ((double)(myRobot.GetSensorBatteryAD1() )/ 4095.0 * 9).ToString("0.00") + "V";
            lblMotorVol.Text = ((double)(myRobot.GetSensorBatteryAD2()) / 4095.0 * 24).ToString("0.00") + "V";
            lblLeftAlarm.Text = (myRobot.GetSensorHumanAlarm1()).ToString();
            lblLeftMotion.Text = (myRobot.GetSensorHumanMotion1()).ToString();
            lblRightAlarm.Text = (myRobot.GetSensorHumanAlarm2()).ToString();
            lblRightMotion.Text = (myRobot.GetSensorHumanMotion2()).ToString();
        }

        private void btnForward_Click(object sender, EventArgs e)
        {
            short speed = (short)trackBar1.Value;
            myRobot.DcMotorVelocityNonTimeCtrAll((short)-speed, speed, -32767, -32767, -32767, -32767);
        }

        private void btnStop_Click(object sender, EventArgs e)
        {
            myRobot.DisableDcMotor(0);
            myRobot.DisableDcMotor(1);
        }

        private void tmrCamera_Tick(object sender, EventArgs e)
        {
            myCamera.TakePhoto();
        }

        private void btnLeft_Click(object sender, EventArgs e)
        {
            short speed = (short)trackBar1.Value;
            myRobot.DcMotorVelocityNonTimeCtrAll(speed, speed, -32767, -32767, -32767, -32767);
        }

        private void btnBack_Click(object sender, EventArgs e)
        {
            short speed = (short)trackBar1.Value;
            myRobot.DcMotorVelocityNonTimeCtrAll(speed, (short)-speed, -32767, -32767, -32767, -32767);
        }

        private void btnRight_Click(object sender, EventArgs e)
        {
            short speed = (short)trackBar1.Value;
            myRobot.DcMotorVelocityNonTimeCtrAll((short)-speed, (short)-speed, -32767, -32767, -32767, -32767);
        }

        private void myRobot_MotorSensorEvent(object sender, EventArgs e)
        {
            lblLeftPos.Text = (myRobot.GetEncoderPulse1()).ToString();
            lblRightPos.Text = (myRobot.GetEncoderPulse2()).ToString();

            lblLeftSpeed.Text = (myRobot.GetEncoderSpeed1()).ToString();
            lblRightSpeed.Text = (myRobot.GetEncoderSpeed2()).ToString();
            lblLeftCurrent.Text = ((double)myRobot.GetMotorCurrent1() / 728.0).ToString("0.00") + "A";
            lblRightCurrent.Text = ((double)myRobot.GetMotorCurrent2() /728.0).ToString("0.00") + "A";
            
        }

        private void myRobot_CustomSensorEvent(object sender, EventArgs e)
        {
            lblIRDis2.Text = (AD2Dis(myRobot.GetCustomAD3())).ToString("0.00") +"m";
            lblIRDis3.Text = (AD2Dis(myRobot.GetCustomAD4())).ToString("0.00")+"m";
            lblIRDis4.Text = (AD2Dis(myRobot.GetCustomAD5())).ToString("0.00")+"m";
            lblIRDis5.Text = (AD2Dis(myRobot.GetCustomAD6())).ToString("0.00")+"m";
            lblIRDis6.Text = (AD2Dis(myRobot.GetCustomAD7())).ToString("0.00")+"m";
            lblIRDis7.Text = (AD2Dis(myRobot.GetCustomAD8())).ToString("0.00")+"m";
            
        }

        private void btnForward1M_Click(object sender, EventArgs e)
        {
            RunDis(1.0);           

        }

        private void btnTurn90_Click(object sender, EventArgs e)
        {
            Turn(90);

        }

        private void btnDemo_Click(object sender, EventArgs e)
        {
            MotorDemoStep = 0;
            RunDis(1.0); //forward 1m
            tmrMotorDemo.Enabled = true;

        }
        private void Turn(double degreee)
        {
            double Dis = WheelDis / 2 * (degreee / 180.0 * Math.PI);
            int diffEncoder = (int)(Dis / (2 * Math.PI * WheelR) * CircleCnt);

            int LeftCmd = int.Parse(lblLeftPos.Text) + diffEncoder;
            if (LeftCmd < 0)
            {
                LeftCmd = 32767 + LeftCmd;
            }
            else if (LeftCmd > 32767)
            {
                LeftCmd = LeftCmd - 32767;
            }

            int RightCmd = int.Parse(lblRightPos.Text) + diffEncoder;
            if (RightCmd < 0)
            {
                RightCmd = 32767 + RightCmd;
            }
            else if (RightCmd > 32767)
            {
                RightCmd = RightCmd - 32767;
            }
            myRobot.SetDcMotorPositionControlPID(0, 1000, 5, 10000);
            myRobot.SetDcMotorPositionControlPID(1, 1000, 5, 10000);
            myRobot.DcMotorPositionTimeCtrAll((short)LeftCmd, (short)RightCmd, -32767, -32767, -32767, -32767, 4000);
        }



        private void RunDis(double Dis)
        {
            Dis = 1.0;        //1m
            int diffEncoder = (int)(Dis / (2 * Math.PI * WheelR) * CircleCnt);

            int LeftCmd = int.Parse(lblLeftPos.Text) - diffEncoder;
            if (LeftCmd < 0)
            {
                LeftCmd = 32767 + LeftCmd;
            }
            else if (LeftCmd > 32767)
            {
                LeftCmd = LeftCmd - 32767;
            }

            int RightCmd = int.Parse(lblRightPos.Text) + diffEncoder;
            if (RightCmd < 0)
            {
                RightCmd = 32767 + RightCmd;
            }
            else if (RightCmd > 32767)
            {
                RightCmd = RightCmd - 32767;
            }

            myRobot.DcMotorPositionTimeCtrAll((short)LeftCmd, (short)RightCmd, -32767, -32767, -32767, -32767, 10000);

        }

        private void tmrMotorDemo_Tick(object sender, EventArgs e)
        {
            MotorDemoStep++;
            if (MotorDemoStep == 1)
                Turn(90);
            else if (MotorDemoStep == 2)
                RunDis(1.0);
            else if (MotorDemoStep == 3)
                Turn(90);
            else if (MotorDemoStep == 4)
                RunDis(1.0);
            else if (MotorDemoStep ==5)
                Turn(90);
            else if (MotorDemoStep == 6)
                RunDis(1.0);
            else if (MotorDemoStep == 7)
                Turn(90);
            else if (MotorDemoStep == 8)
            {
                   //stop here
                tmrMotorDemo.Enabled = false;
                MotorDemoStep = -1;
            }

        }

        private void btnServoDemo_Click(object sender, EventArgs e)
        {
            ServoDemoStep = 0;
            //3500, 3200, yo can modify the value based on your robot
            myRobot.ServoTimeCtrAll(3500, 3200, -32767, -32767, -32767, -32767, 1500);
            tmrServoDemo.Enabled = true;

        }

        private void tmrServoDemo_Tick(object sender, EventArgs e)
        {
            ServoDemoStep++;
            if (ServoDemoStep == 1)
                myRobot.ServoTimeCtrAll(4200, 3200, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 2)
                myRobot.ServoTimeCtrAll(2400, 3200, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 3)
                myRobot.ServoTimeCtrAll(4200, 3200, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 4)
                myRobot.ServoTimeCtrAll(2400, 3200, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 5)
                myRobot.ServoTimeCtrAll(3500, 3200, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 6)
                myRobot.ServoTimeCtrAll(3500, 4400, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 7)
                myRobot.ServoTimeCtrAll(3500, 2400, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 8)
                myRobot.ServoTimeCtrAll(3500, 4400, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 9)
                myRobot.ServoTimeCtrAll(3500, 2400, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 10)
                myRobot.ServoTimeCtrAll(3500, 3200, -32767, -32767, -32767, -32767, 1500);
            else if (ServoDemoStep == 1)
            {
                ServoDemoStep = -1;
                tmrServoDemo.Enabled = false;
            }

        }

        private double AD2Dis(Int16 IRValue)
        {
            double temp = 0;
            double IRAD2Distance = 0;

            temp = 21.6 / ((double)IRValue * 3 / 4028 - 0.17);

            // IR range 10-80cm
            if ((temp > 80) || (temp < 0))
                IRAD2Distance = 0.81;
            else if ((temp < 10) && (temp > 0))
                IRAD2Distance = 0.09;
            else
                IRAD2Distance = temp / 100;
            return IRAD2Distance;
        }


        private void trackBar2_Scroll(object sender, EventArgs e)
        {
            Int16 cmdValue = (short)trackBar2.Value;
            myRobot.ServoTimeCtr(1, (short)cmdValue, 1000);

        }

        private void trackBar3_Scroll(object sender, EventArgs e)
        {
            Int16 cmdValue = (short)trackBar3.Value;
            myRobot.ServoTimeCtr(0, (short)cmdValue, 1000);

        }

        private void btnSaveImage_Click(object sender, EventArgs e)
        {
            myCamera.SavePhotoAsBMP(Environment.CurrentDirectory + "photo.bmp");

        }

        private void btnPlay_Click(object sender, EventArgs e)
        {
            if (btnPlay.Text == "Play Music")
            {

                openFileDialog1.InitialDirectory = ".\\";
                openFileDialog1.Filter = "wave Files (*.wav)|*.wav";
                openFileDialog1.Title = "Type File";
                //openFileDialog1.ShowDialog();

                if (openFileDialog1.ShowDialog() != DialogResult.OK)
                    return;
                else
                {
                    myCamera.PlayAudioFile(openFileDialog1.FileName);
                    btnPlay.Text = "Stop Music";

                }

            }
            else
            {
                myCamera.StopAudioPlay();
                btnPlay.Text = "Play Music";
            }
        }

        private void tmrJoyStickPoll_Tick(object sender, EventArgs e)
        {
            GetData();
        }

        private void btnJoyStick_Click(object sender, EventArgs e)
        {
            if (btnJoyStick.Text == "Enable Joystick")
            {

                blnJoyStick = true;
                btnJoyStick.Text = "Disable Joystick";
            }
            else{
                blnJoyStick = false ;
                btnJoyStick.Text = "Enable Joystick";
            }
        }

        private void btnListening_Click(object sender, EventArgs e)
        {
            if (btnListening.Text == "Start Listen")
            {
                btnListening.Text = "Stop Listen";
                blnListen = true;
                //prepare buffer
                Stop();
                try
                {
                    WaveLib.WaveFormat fmt = new WaveLib.WaveFormat(8000, 16, 1);
                    m_Player = new WaveLib.WaveOutPlayer(-1, fmt, 8192, 3, new WaveLib.BufferFillEventHandler(Filler));
                    myCamera.StartRecord(1);     //about 256ms, 4096 data
                }
                catch
                {
                    Stop();
                    //throw;
                }
                
            }
            else
            {
                btnListening.Text = "Start Listen";
                blnListen = false;
                myCamera.StopRecord();
                Stop();
            }

        }

        private void Filler(IntPtr data, int size)
        {
            if (m_PlayBuffer == null || m_PlayBuffer.Length < size)
                m_PlayBuffer = new byte[size];
            if (m_Fifo.Length >= size)
                m_Fifo.Read(m_PlayBuffer, 0, size);
            else
                for (int i = 0; i < m_PlayBuffer.Length; i++)
                    m_PlayBuffer[i] = 0;
            System.Runtime.InteropServices.Marshal.Copy(m_PlayBuffer, 0, data, size);
        }

        private void Stop()
        {
            if (m_Player != null)
                try
                {
                    m_Player.Dispose();
                }
                finally
                {
                    m_Player = null;
                }
         
            m_Fifo.Flush(); // clear all pending data
        }

        private void myRobot_VoiceSegmentEvent(object sender, EventArgs e)
        {
            
        }

        private void Form1_Closing(object sender, System.ComponentModel.CancelEventArgs e)
        {
            myRobot.StopRecord();
            tmrCamera.Enabled = false;
            tmrJoyStickPoll.Enabled = false;
            // Unacquire all DirectInput objects.
            if (null != applicationDevice)
                applicationDevice.Unacquire();

            Stop();
        }

        private void myCamera_VoiceSegmentEvent(object sender, EventArgs e)
        {
            if (blnListen)
            {
                //m_Fifo.Flush();
                int size = myCamera.GetVoiceSegLength();
                IntPtr data = (IntPtr)myCamera.GetVoiceSegment();
                if (m_RecBuffer == null || m_RecBuffer.Length < size)
                    m_RecBuffer = new byte[size];
                System.Runtime.InteropServices.Marshal.Copy(data, m_RecBuffer, 0, size);
                m_Fifo.Write(m_RecBuffer, 0, m_RecBuffer.Length);
            }
        }

        private void trackBar4_Scroll(object sender, EventArgs e)
        {
            Int16 cmdValue = (short)trackBar4.Value;
            myRobot.ServoTimeCtr(2, (short)cmdValue, 1000);
        }

        private void trackBar5_Scroll(object sender, EventArgs e)
        {
            Int16 cmdValue = (short)trackBar5.Value;
            myRobot.ServoTimeCtr(5, (short)cmdValue, 1000);

        }

        private void trackBar6_Scroll(object sender, EventArgs e)
        {
            Int16 cmdValue = (short)trackBar6.Value;
            myRobot.ServoTimeCtr(4, (short)cmdValue, 1000);
        }

        private void btnDisServo_Click(object sender, EventArgs e)
        {
            myRobot.DisableServo(0);
            myRobot.DisableServo(1);
            myRobot.DisableServo(2);
            myRobot.DisableServo(3);
            myRobot.DisableServo(4);
            myRobot.DisableServo(5);
        }

       
        private void btnTakephoto_Click(object sender, EventArgs e)
        {
            myCamera.TakePhoto();
        }

        private void btnExit_Click(object sender, EventArgs e)
        {
            Close();
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            System.Diagnostics.Process.Start("http://www.drrobot.com");
        }

       

       
    }
}